#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: MrQ
# date: 2023/08/02

# Comments directed to http://gimpchat.com

# Inspired by Tim Tran script "Map Fold and Tom Bland video tutorial".

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.


# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial relase

import math
from gimpfu import *

####################################################################################################################
################# START gegl_command by Kevin Payne & Claude Lion [cli345] #########################################
####################################################################################################################

from sys import platform
from ctypes import *

def load_library (library_name):
	if platform == "linux" or platform == "linux2":
		library_name = library_name + '.so.0'
	elif platform == "win32":
		from ctypes.util import find_library
		library_name = find_library (library_name + "-0")
	else:
		raise BaseException ("TODO")
	return CDLL (library_name)

gimpC = load_library ('libgimp-2.0')


def gegl_command_fold(image, drawable, gegl_graph_string):
	
	class GeglBuffer(Structure):
		pass
	drawable_id = drawable.ID
	
	gegl = load_library ('libgegl-0.4')
	gegl.gegl_init (None, None)
	gimpC.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
	gimpC.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

	x = c_int (); y = c_int (); w = c_int (); h = c_int ()
	non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
	args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
	
	if non_empty:
		source = gimpC.gimp_drawable_get_buffer (drawable_id)
		target = gimpC.gimp_drawable_get_shadow_buffer (drawable_id)
		gegl.gegl_render_op (source, target, "gegl:gegl", *args)
		gegl.gegl_buffer_flush (target)
		gimpC.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
		gimpC.gimp_drawable_update (drawable_id, x, y, w, h)
		gimpC.gimp_displays_flush ()

####################################################################################################
################# END gegl_command #################################################################
####################################################################################################
def rgb_to_hex(rgb_color):
	r = rgb_color[0]
	g = rgb_color[1]
	b = rgb_color[2]
	return '#%02x%02x%02x' % (r,g,b)
##########################################################################################################
def copy_layers_iteration(image, drawable, iterations):
    drawable = pdb.gimp_image_get_active_layer(image)
    for item in range(0,int(iterations)):
        copy=pdb.gimp_layer_copy(drawable,1)
        pdb.gimp_image_add_layer(image,copy,0)
        pdb.gimp_item_set_name(copy, drawable.name +" - " + str(item+1))
##########################################################################################################
		
def MrQ_Fold_image(img, layer, fold, First_fold, Second_fold, Third_fold, Fourth_fold, Five_fold, y_magnitud, pp, draw_fold, color_stroke, angle, run_long_s, style, angle_ls, length_ls, color_ls,):

    width = pdb.gimp_image_width(img)
    height = pdb.gimp_image_height(img)
	
    active_layer = pdb.gimp_image_get_active_layer(img)		
    new_image = pdb.gimp_image_new(width, height, 0)
    display = pdb.gimp_display_new(new_image)
    active = pdb.gimp_layer_new_from_drawable(layer, new_image)
    pdb.gimp_image_insert_layer(new_image, active, None, 0)		
    active_new = pdb.gimp_image_get_active_layer(new_image)	
    active_new.name = active_layer.name
	
    #Undo group start
    pdb.gimp_image_undo_group_start(new_image)
	
########## remove the extension from the layer name ########
    sourcename = active_layer.name
    layer = active_new
    if '.' in sourcename:
        split = sourcename.split(".", 1)
        sourcename = split[0]
        active_new.name = sourcename	
		
    y_shear = (height * y_magnitud) / 100
################# end remove  ##############################      

    layer=pdb.gimp_image_get_active_layer(new_image)	
    pdb.gimp_layer_add_alpha(layer)
    if fold==0:    
     
        copy_layers_iteration(new_image, layer, 1)
		
        #First fold			
        pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, width /2, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[0])
        pdb.gimp_selection_none(new_image)			
        pdb.gimp_image_select_item(new_image, 2, new_image.layers[0])		
        if First_fold==0:	# " = "		
            pdb.gimp_selection_none(new_image)
        if First_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /2, 0-y_shear,   0,  height-y_shear,  width /2,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /2, 0+y_shear,   0,  height+y_shear,  width /2,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /2, 0,   0,  height+y_shear,    width /2,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==4:	# " = / "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /2, -y_shear,   0,  height,    width /2,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])			
        if First_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /2, 0,   0,  height-y_shear,  width /2,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /2, y_shear,   0,  height,    width /2,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])				

        pdb.gimp_selection_none(new_image)
		
		# Second_fold		
        pdb.gimp_image_select_rectangle(new_image, 2,  width/2, 0, width /2, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[1])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[1])

        if Second_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Second_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width/2, 0-y_shear,   	width, 0-y_shear, 	width/2, height-y_shear,    width, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width/2, 0+y_shear,   	width, 0+y_shear, 	width/3, height+y_shear,    width, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /2, 0+y_shear,    width, 0,   width /3,  height+y_shear,   width ,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /2, 0,            width , -y_shear,   width /2,  height,   width,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /2, 0-y_shear,    width , 0,   width /2,  height-y_shear,  width,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /2, 0,             width , y_shear,   width /2,  height,   width,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])		
    if fold==1:    
     
        copy_layers_iteration(new_image, layer, 2)
		
        #First fold			
        pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, width /3, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[0])
        pdb.gimp_selection_none(new_image)			
        pdb.gimp_image_select_item(new_image, 2, new_image.layers[0])		
        if First_fold==0:	# " = "		
            pdb.gimp_selection_none(new_image)
        if First_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /3, 0-y_shear,   0,  height-y_shear,  width /3,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /3, 0+y_shear,   0,  height+y_shear,  width /3,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /3, 0,   0,  height+y_shear,    width /3,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==4:	# " = / "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /3, -y_shear,   0,  height,    width /3,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])			
        if First_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /3, 0,   0,  height-y_shear,  width /3,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /3, y_shear,   0,  height,    width /3,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])				

        pdb.gimp_selection_none(new_image)
		
		# Second_fold		
        pdb.gimp_image_select_rectangle(new_image, 2,  width/3, 0, width /3, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[1])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[1])

        if Second_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Second_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width/3, 0-y_shear,   	2*width/3, 0-y_shear, 	width/3, height-y_shear,    2*width/3, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width/3, 0+y_shear,   	2*width/3, 0+y_shear, 	width/3, height+y_shear,    2*width/3, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /3, 0+y_shear,    2*width /3, 0,   width /3,  height+y_shear,   2* width /3,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /3, 0,    2*width /3, -y_shear,   width /3,  height,   2* width /3,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /3, 0-y_shear,    2*width /3, 0,   width /3,  height-y_shear,  2*width /3,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[1],width /3, 0,    2*width /3, y_shear,   width /3,  height,   2* width /3,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])			

        #Third fold
        pdb.gimp_selection_none(new_image)		
        pdb.gimp_image_select_rectangle(new_image, 2,  2*width /3, 0, width /3, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[2])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[2])		
		
        if Third_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Third_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width/3, 0-y_shear,   	width, 0-y_shear, 	2*width/3, height-y_shear,    width, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[2])        
        if Third_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width/3, 0+y_shear,   	width, 0+y_shear, 	2*width/3, height+y_shear,    width, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[2])        
        if Third_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width /3, 0+y_shear,    width, 0,   2*width /3,  height+y_shear,   width,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[2],2* width /3, 0,   width, -y_shear,   2*width /3,  height,   width,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width /3, 0-y_shear,    width, 0,   2*width /3,  height-y_shear,   width,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[2],2*width /3, 0,    width, y_shear,   2*width /3,  height,   width,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])		
     
		      

######################## Four fold option #############################################################################  
 # 0 " =", 1 " = Up", 2 " = Down", 3 " / ", 4 " \ "   
    if fold==2:
        copy_layers_iteration(new_image, layer, 3)
        #First fold		
        pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, width /4, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[0])
        pdb.gimp_selection_none(new_image)			
        pdb.gimp_image_select_item(new_image, 2, new_image.layers[0])		

        if First_fold==0:	# " = "		
            pdb.gimp_selection_none(new_image)
        if First_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /4, 0-y_shear,   0,  height-y_shear,  width /4,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /4, 0+y_shear,   0,  height+y_shear,  width /4,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /4, 0,   0,  height+y_shear,    width /4,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==4:	# " = / "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /4, -y_shear,   0,  height,    width /4,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])			
        if First_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /4, 0,   0,  height-y_shear,  width /4,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /4, y_shear,   0,  height,    width /4,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])	
			
          
        #Second fold
        pdb.gimp_image_select_rectangle(new_image, 2,  width/4, 0, width /4, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[1])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[1])
		
        if Second_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Second_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /4, 0-y_shear,    width /2, 0-y_shear, 	width /4, height-y_shear,    width /2, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /4, 0+y_shear,    width /2, 0+y_shear, 	width /4, height+y_shear,    width /2, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /4, 0+y_shear,    width /2, 0,         width /4,  height+y_shear,   width /2,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /4, 0,            width /2, -y_shear,   width /4,  height,   width /2,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /4, 0-y_shear,    width /2, 0,          width /4,  height-y_shear,  width /2,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /4, 0,            width /2, y_shear,     width /4,  height,   width /2,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])	
          
        #Third fold
        pdb.gimp_selection_none(new_image)		
        pdb.gimp_image_select_rectangle(new_image, 2,  width /2, 0, width /4, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[2])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[2])		
		
        if Third_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Third_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[2], width /2, 0-y_shear,   	3*width/4, 0-y_shear, 	   width /2, height-y_shear,       3*width/4, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[2])        
        if Third_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[2], width /2, 0+y_shear,   	3*width/4, 0+y_shear, 	   width /2, height+y_shear,       3*width/4, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[2])        
        if Third_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[2], width /2, 0+y_shear,       3*width/4, 0,              width /2,  height+y_shear,      3*width/4,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[2], width /2, 0,              3*width/4, -y_shear,        width /2,  height,              3*width/4,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[2], width /2, 0-y_shear,      3*width/4, 0,               width /2,  height-y_shear,       3*width/4,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[2],width /2, 0,               3*width/4, y_shear,         width /2, height,                3*width/4,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])	
       
        #Fourth fold
        pdb.gimp_selection_none(new_image)		
        pdb.gimp_image_select_rectangle(new_image, 2,  3*width /4, 0, width /4, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[3])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[3])		
		
        if Fourth_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Fourth_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /4, 0-y_shear,   	width, 0-y_shear, 	      3*width /4, height-y_shear,        width, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[3])        
        if Fourth_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /4, 0+y_shear,   	width, 0+y_shear, 	      3*width /4, height+y_shear,        width, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[3])        
        if Fourth_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /4, 0+y_shear,    width, 0,                  3*width /4, height+y_shear,        width,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])
        if Fourth_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /4, 0,            width, -y_shear,            3*width /4, height,                width,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])
        if Fourth_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /4, 0-y_shear,    width, 0,                   3*width /4,  height-y_shear,       width,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])
        if Fourth_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[3],   3*width /4, 0,           width, y_shear,             3*width /4,  height,               width,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])
######################## Four fold option #############################################################################  
 # 0 " =", 1 " = Up", 2 " = Down", 3 " / ", 4 " \ "   
    if fold==3:
        copy_layers_iteration(new_image, layer, 4)
        #First fold		
        pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, width /5, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[0])
        pdb.gimp_selection_none(new_image)			
        pdb.gimp_image_select_item(new_image, 2, new_image.layers[0])		

        if First_fold==0:	# " = "		
            pdb.gimp_selection_none(new_image)
        if First_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /5, 0-y_shear,   0,  height-y_shear,  width /5,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /5, 0+y_shear,   0,  height+y_shear,  width /5,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0+y_shear,    width /5, 0,   0,  height+y_shear,    width /5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==4:	# " = / "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /5, -y_shear,   0,  height,    width /5,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])			
        if First_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0-y_shear,    width /5, 0,   0,  height-y_shear,  width /5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])
        if First_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[0], 0, 0,    width /5, y_shear,   0,  height,    width /5,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[0])	
			
          
        #Second fold
        pdb.gimp_image_select_rectangle(new_image, 2,  width/5, 0, width /5, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[1])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[1])
		
        if Second_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Second_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /5, 0-y_shear,    2*width /5, 0-y_shear, 	width /5, height-y_shear,    2*width /5, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /5, 0+y_shear,    2*width /5, 0+y_shear, 	width /5, height+y_shear,    2*width /5, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[1])        
        if Second_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /5, 0+y_shear,    2*width /5, 0,            width /5,  height+y_shear,      2*width /5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /5, 0,            2*width /5, -y_shear,      width /5,  height,             2*width /5,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /5, 0-y_shear,    2*width /5, 0,             width /5,  height-y_shear,      2*width /5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])
        if Second_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[1], width /5, 0,           2*width /5, y_shear,        width /5,  height,             2*width /5,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[1])	
          
        #Third fold
        pdb.gimp_selection_none(new_image)		
        pdb.gimp_image_select_rectangle(new_image, 2,  2*width /5, 0, width /5, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[2])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[2])		
		
        if Third_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Third_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width /5, 0-y_shear,   	3*width/5, 0-y_shear, 	   2*width /5, height-y_shear,       3*width/5, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[2])        
        if Third_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width /5, 0+y_shear,   	3*width/5, 0+y_shear, 	   2*width /5, height+y_shear,       3*width/5, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[2])        
        if Third_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width /5, 0+y_shear,       3*width/5, 0,              2*width /5,  height+y_shear,      3*width/5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width /5, 0,              3*width/5, -y_shear,        2*width /5,  height,              3*width/5,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[2], 2*width /5, 0-y_shear,      3*width/5, 0,               2*width /5,  height-y_shear,       3*width/5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])
        if Third_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[2],2*width /5, 0,               3*width/5, y_shear,         2*width /5, height,                3*width/5,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[2])	
       
        #Fourth fold
        pdb.gimp_selection_none(new_image)		
        pdb.gimp_image_select_rectangle(new_image, 2,  3*width /5, 0, width /5, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[3])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[3])		
		
        if Fourth_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Fourth_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /5, 0-y_shear,   	4*width /5, 0-y_shear, 	      3*width /5, height-y_shear,        4*width/5, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[3])        
        if Fourth_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /5, 0+y_shear,   	4*width /5, 0+y_shear, 	      3*width /5, height+y_shear,        4*width/5, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[3])        
        if Fourth_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /5, 0+y_shear,    4*width /5, 0,                  3*width /5, height+y_shear,        4*width/5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])
        if Fourth_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /5, 0,            4*width /5, -y_shear,            3*width /5, height,               4* width/5,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])
        if Fourth_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[3],  3*width /5, 0-y_shear,    4*width /5, 0,                   3*width /5,  height-y_shear,       4*width/5,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])
        if Fourth_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[3],   3*width /5, 0,           4*width /5, y_shear,             3*width /5,  height,              4* width/5,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[3])        
        #Create a fold shadow effect

        pdb.gimp_selection_none(new_image)		
        pdb.gimp_image_select_rectangle(new_image, 2,  4*width /5, 0, width /5, height)
        pdb.gimp_selection_invert(new_image)
        pdb.gimp_drawable_edit_clear(new_image.layers[4])
        pdb.gimp_selection_none(new_image)
        sel1=pdb.gimp_image_select_item(new_image, 2, new_image.layers[4])		
		
        if Five_fold==0:		
            pdb.gimp_selection_none(new_image)
        if Five_fold==1:	# " = Up"
            pdb.gimp_item_transform_perspective(new_image.layers[4],  4*width /5, 0-y_shear,   	width, 0-y_shear, 	     4* width/5, height-y_shear,        width, height-y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[4])        
        if Five_fold==2:	# " = Down"
            pdb.gimp_item_transform_perspective(new_image.layers[4],  4*width /5, 0+y_shear,   	width, 0+y_shear, 	      4*width/5, height+y_shear,        width, height+y_shear )
            pdb.gimp_floating_sel_anchor(new_image.layers[4])        
        if Five_fold==3:	# " / = "
            pdb.gimp_item_transform_perspective(new_image.layers[4],  4*width /5, 0+y_shear,    width, 0,                  4*width/5, height+y_shear,        width,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[4])
        if Five_fold==4:	# " = / "			
            pdb.gimp_item_transform_perspective(new_image.layers[4],  4*width /5, 0,            width, -y_shear,            4*width/5, height,                width,  height-y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[4])
        if Five_fold==5:	# " \ = "
            pdb.gimp_item_transform_perspective(new_image.layers[4],  4*width /5, 0-y_shear,    width, 0,                   4*width/5, height-y_shear,       width,  height)
            pdb.gimp_floating_sel_anchor(new_image.layers[4])
        if Five_fold==6:	# " = \ "
            pdb.gimp_item_transform_perspective(new_image.layers[4],   4*width /5, 0,           width, y_shear,             4*width/5,  height,               width,  height+y_shear)
            pdb.gimp_floating_sel_anchor(new_image.layers[4])        
        #Create a fold shadow effect
    pdb.gimp_selection_none(new_image)		
    pdb.gimp_image_resize_to_layers(new_image)    

	
    if draw_fold==1:
        for layer in new_image.layers:
            pdb.gimp_selection_none(new_image)		
            sel2=pdb.gimp_image_select_item(new_image, 2, layer)
            pdb.plug_in_sel2path(new_image, sel2)			
            active_vectors = pdb.gimp_image_get_active_vectors(new_image)	
            W = pdb.gimp_image_width(new_image)
            H = pdb.gimp_image_height(new_image)
            pdb.gimp_context_set_foreground(color_stroke)
            pdb.gimp_context_set_brush("2. Hardness 100");
            pdb.gimp_context_set_brush_size(2)			
            pdb.gimp_context_set_brush_hardness(1);
            pdb.gimp_context_set_brush_spacing(0.01);
            pdb.gimp_context_set_paint_method("gimp-paintbrush")
            pdb.gimp_context_set_dynamics("Dynamics Off") 
			
            stroke_b = gimp.Layer(new_image, "Stroke", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
            new_image.add_layer(stroke_b, -1)
            pdb.gimp_drawable_edit_stroke_item(stroke_b ,active_vectors)
            pdb.gimp_image_remove_vectors(new_image, active_vectors)	
            pdb.gimp_image_raise_item_to_top(new_image, stroke_b)	
			
    pdb.gimp_selection_none(new_image)		
    fold=pdb.gimp_layer_new_from_visible(new_image, new_image, "Fold")
    pdb.gimp_image_insert_layer(new_image, fold, None, 0)
	

		
		
    if angle !=0:
        w = pdb.gimp_drawable_width(fold)
        h = pdb.gimp_drawable_height(fold)
        pdb.gimp_item_transform_perspective(fold, -angle, 0,   w+angle, 0,   0, h,    w, h)
        pdb.gimp_image_resize_to_layers(new_image)	

    pdb.gimp_image_resize(new_image,new_image.width+2*pp,new_image.height+2*pp, pp, pp)
    pdb.gimp_layer_resize_to_image_size(fold)
	
    if run_long_s==1:

        fold=pdb.gimp_image_get_active_layer(new_image)
        copy = pdb.gimp_layer_copy(fold, TRUE)
        pdb.gimp_image_insert_layer(new_image, copy, None, -1)
        pdb.gimp_item_set_name(copy, "Long Shadow")
        pdb.gimp_layer_resize_to_image_size(copy)
        color_to_gegl=rgb_to_hex(color_ls)
		
        if style==0:
            style_ls="finite"
        if style==1:
            style_ls="infinite"
        if style==2:
            style_ls="fading"
        if style==3:
            style_ls="fading-fixed-length"			
			
        gegl_command_fold(new_image, copy, "long-shadow style=%s angle=%d length=%d color=%s" % (style_ls,angle_ls, length_ls,color_to_gegl))
		

	
    active_layer = pdb.gimp_image_get_active_layer(new_image)
	
    for layer in new_image.layers:
        pdb.gimp_image_set_active_layer(new_image,layer)
        pdb.gimp_item_set_visible(layer, 0)	
    pdb.gimp_item_set_visible(fold, 1)	
	
    if run_long_s==1:
        pdb.gimp_layer_resize_to_image_size(copy)
        pdb.gimp_item_set_visible(copy, 1)
		
    pdb.gimp_displays_flush()  

	#Undo group end
    pdb.gimp_image_undo_group_end(new_image)  
	
register(
    "MrQ_Fold_image",
    "Create Fold Brochure Effect.\nVer.0.1.",
    "Create Fold Brochure Effect",
    "MrQ",
    "GPL",
    "2023",
    "Fold image...",
    "",
    [	
		(PF_IMAGE,    "new_image",   "Source image", None),
		(PF_DRAWABLE, "layer",       "Source layer", None),
		(PF_OPTION,   "fold",        "Fold Selection:", 0 ,["2", "3", "4","5"]),
		(PF_OPTION,   "First_fold",  "First fold:",     0 ,[ " =", " = Up", " = Down", " / = ", " = / ", " \ = ", " = \ "]),	
		(PF_OPTION,   "Second_fold", "Second fold:",    0 ,[ " =", " = Up", " = Down", " / = ", " = / ", " \ = ", " = \ "]),	
		(PF_OPTION,   "Third_fold",  "Third fold:",     0 ,[ " =", " = Up", " = Down", " / = ", " = / ", " \ = ", " = \ "]),
		(PF_OPTION,   "Fourth_fold", "Fourth fold:",    0 ,[ " =", " = Up", " = Down", " / = ", " = / ", " \ = ", " = \ "]),
		(PF_OPTION,   "Five_fold",   "Fifth fold:",     0 ,[ " =", " = Up", " = Down", " / = ", " = / ", " \ = ", " = \ "]),		
		(PF_SPINNER,  "y_magnitud",  "Shear Magnitude [% H]", 15, (5, 50, 1)),
		(PF_SPINNER,  "pp",          "Empty border [px]",     30, (0, 1000, 1)),
		(PF_TOGGLE , "draw_fold" ,   "Draw the fold edges?", 1 ,["YES","NO"]),
		(PF_COLOR,    "color_stroke", "Draw Color: ",(255,255,255)),
		(PF_SPINNER, "angle",         "Transform Perspective ON TOP  \n    - Squeeze (px) Stretch +", 0, (-500, 500, 1)),
		(PF_TOGGLE , "run_long_s" ,   "Create Shadow (Gegl)?" , 0 ,["YES","NO"]),
		(PF_OPTION,  "style",         "Style:", 3, ["Finite","Infinite","Fading","Fading (fixed length)"]),		
		(PF_SPINNER, "angle_ls",      "Angle: ", 45, (-180, 180, 1)),
		(PF_SPINNER, "length_ls",     "Length:", 20, (0, 1000, 1)),		
		(PF_COLOR,   "color_ls",      "Color:", (0,0,0)),		
    ],
    [],
    MrQ_Fold_image, menu="<Image>/Filters/Map"),   
main()	